#include "common.h"
#include <stdio.h>
#include <stdarg.h>

//int _fltused = 0x1234;
typedef struct tagSTRINGS{
	UINT id;
	LPTSTR str;
	struct tagSTRINGS* next;
} STRINGS;
static STRINGS* LoadedStrings = NULL;

#if 0
void DebugBox(char *str, ...)
{
	va_list varg;
	char buf[256];

	va_start(varg, str);
	vsprintf(buf, str, varg);
	va_end(varg);

	MessageBox(NULL, buf, "DebugMessage", MB_OK | MB_ICONINFORMATION);

	return;
}
#endif

void* halloc(DWORD bytes)
{
	return HeapAlloc(GetProcessHeap(), 0, bytes);
}

void* hrealloc(void* ptr, DWORD bytes)
{
	return HeapReAlloc(GetProcessHeap(), 0, ptr, bytes);
}

BOOL hfree(void* ptr)
{
	return HeapFree(GetProcessHeap(), 0, ptr);
}

char* ReadLine(HANDLE hf)
{
	char* str;
	DWORD size = BUFFBLOCK;
	DWORD read;
	DWORD cur;

	if(!(str = halloc(size)))
		return NULL;

	for(cur = 0;;)
	{
		if(!ReadFile(hf, str + cur, 1, &read, NULL))
		{
			hfree(str);
			return NULL;
		}

		if(!read)	// EOF
			break;

		if(*(str + cur) == 0x0D || *(str + cur) == 0x0A)
		{
			BYTE test;

			if(!ReadFile(hf, &test, 1, &read, NULL))
			{
				hfree(str);
				return NULL;
			}
			if(test != 0x0A)	// if not CRLF(unix/mac)...really happen this?
			{
				SetFilePointer(hf, -1, NULL, FILE_CURRENT);
			}

			break;
		}

		cur++;
		if(cur >= size)
		{	// resize "str"
			char* newstr;

			size += BUFFBLOCK;
			if(!(newstr = hrealloc(str, size)))
			{
				hfree(str);
				return NULL;
			}
			str = newstr;
		}
	}

	*(str + cur) = TEXT('\0');

	{
		char* newstr;

		if(!(newstr = hrealloc(str, cur + 1)))
			return newstr;
		str = newstr;
	}

	return str;
}

LPCTSTR basename(LPCTSTR str)
{
	LPCTSTR last = str;

	while(*str)
	{
		if(*str == TEXT('\\'))
			last = str + 1;
		str = CharNext(str);
	}

	return last;
}

LPCTSTR lastchar(LPCTSTR str, TCHAR chr)
{
	LPCTSTR last = NULL;

	while(*str)
	{
		if(*str == chr)
			last = str;
		str = CharNext(str);
	}

	return last;
}

BOOL addEndYen(LPTSTR str)
{
	LPTSTR p;
	int chrs, lastyen;

	for(p = str, chrs = 0, lastyen = 0; *p; p = CharNext(p), chrs++)
	{
		if(*p == TEXT('\\'))
			lastyen = chrs;
	}

	if(lastyen < chrs - 1)
	{
		lstrcat(str, TEXT("\\"));
		return TRUE;
	}

	return FALSE;
}

BOOL cutEndYen(LPTSTR str)
{
	LPTSTR p, lyp;
	int chrs, lastyen;

	for(p = str, chrs = 0, lastyen = 0; *p; p = CharNext(p), chrs++)
	{
		if(*p == TEXT('\\'))
		{
			lastyen = chrs;
			lyp = p;
		}
	}

	if(lastyen == chrs - 1)
	{
		*lyp = TEXT('\0');
		return TRUE;
	}

	return FALSE;
}

LPCTSTR lstrchr(LPCTSTR str, TCHAR chr)
{
	LPCTSTR p;

	for(p = str; !chr || *p; p = CharNext(p))
	{
		if(*p == chr)
			return p;
	}

	return NULL;
}

LPCTSTR lstrchri(LPCTSTR str, TCHAR chr)
{
	LPCTSTR p;

	chr = TOLOWER(chr);

	for(p = str; !chr || *p; p = CharNext(p))
	{
		if(TOLOWER(*p) == chr)
			return p;
	}

	return NULL;
}

LPTSTR GetLine(LPTSTR *str)
{
	LPTSTR first = *str;
	LPTSTR p = *str;

	if(!*p)
		return NULL;

	for(; *p; p = CharNext(p))
	{
		if(*p == TEXT('\n'))
			break;
		if(*p == TEXT('\r'))
		{
			if(*CharNext(p) == TEXT('\n'))
			{
				LPTSTR q = p;
				p = CharNext(p);
				*q = TEXT('\0');	// kill '\r'
			}
			break;
		}
	}

	if(!*p)	// oops, crashed to NULL(End of string)!
		return NULL;

	*p = TEXT('\0');

	*str = p + 1;

	return first;
}

LPCTSTR SkipLine(LPCTSTR *str)
{
	LPCTSTR first = *str;
	LPCTSTR p = *str;

	if(!*p)
		return NULL;

	for(; *p; p = CharNext(p))
	{
		if(*p == TEXT('\n'))
			break;
		if(*p == TEXT('\r'))
		{
			if(*CharNext(p) == TEXT('\n'))
				p = CharNext(p);
			break;
		}
	}

	*str = p + 1;

	return first;
}

BOOL makedir(LPCTSTR dir)
{
	TCHAR precurdir[MAX_PATH];
	TCHAR tdir[MAX_PATH];
	LPTSTR p;

	GetCurrentDirectory(sizeof(precurdir), precurdir);

	for(p = tdir; ; )
	{
		lstrcpy(tdir, dir);
		addEndYen(tdir);

		if(!(p = (LPTSTR)lstrchr(p, TEXT('\\'))))
			break;

		p++;
		*p = TEXT('\0');

		CreateDirectory(tdir, NULL);
	}

	SetCurrentDirectory(precurdir);

	return FALSE;
}

BOOL newString(LPTSTR* buf, UINT* bufsize)
{
	*bufsize = BUFFBLOCK;
	if(!(*buf = halloc(*bufsize)))
		return FALSE;
	*buf[0] = TEXT('\0');

	return TRUE;
}

BOOL setString(LPTSTR* buf, UINT* bufsize, LPCTSTR string)
{
	while(*bufsize < (lstrlen(string) + 1) * sizeof(TCHAR))
	{
		*bufsize += BUFFBLOCK;
		if(!(*buf = hrealloc(*buf, *bufsize)))
			return FALSE;
	}

	lstrcpy(*buf, string);

	return TRUE;
}

BOOL appendString(LPTSTR* buf, UINT* bufsize, LPCTSTR string)
{
	while(*bufsize < (lstrlen(*buf) + lstrlen(string) + 1) * sizeof(TCHAR))
	{
		*bufsize += BUFFBLOCK;
		if(!(*buf = hrealloc(*buf, *bufsize)))
			return FALSE;
	}

	lstrcat(*buf, string);

	return TRUE;
}

BOOL appendChar(LPTSTR* buf, UINT* bufsize, TCHAR chr)
{
	TCHAR string[2];
	string[0] = chr;
	string[1] = TEXT('\0');

	while(*bufsize < (lstrlen(*buf) + lstrlen(string) + 1) * sizeof(TCHAR))
	{
		*bufsize += BUFFBLOCK;
		if(!(*buf = hrealloc(*buf, *bufsize)))
			return FALSE;
	}

	lstrcat(*buf, string);

	return TRUE;
}

int lstrcmpn(LPCTSTR str1, LPCTSTR str2, int len)
{
	LPTSTR s1, s2;
	int ret;

	// while(*s1==*s2){s1++;s2++;}ƂĂ ővȂ̂ 킩Ȃ
	// ̕@̓RXg肷B

	s1 = halloc((len + 1) * sizeof(TCHAR));
	s2 = halloc((len + 1) * sizeof(TCHAR));

	lstrcpyn(s1, str1, len);
	lstrcpyn(s2, str2, len);

	ret = lstrcmp(s1, s2);

	hfree(s1);
	hfree(s2);

	return ret;
}

int lstrcmpni(LPCTSTR str1, LPCTSTR str2, int len)
{
	LPTSTR s1, s2;
	int ret;

	// while(*s1==*s2){s1++;s2++;}ƂĂ ővȂ̂ 킩Ȃ
	// ̕@̓RXg肷B

	s1 = halloc((len + 1) * sizeof(TCHAR));
	s2 = halloc((len + 1) * sizeof(TCHAR));

	lstrcpyn(s1, str1, len);
	lstrcpyn(s2, str2, len);

	ret = lstrcmpi(s1, s2);

	hfree(s1);
	hfree(s2);

	return ret;
}

// Win95 NO OSRłLoadStringAcchMax𕶎ƂČoO邽ߎʁB
LPCTSTR AllocLoadString(UINT id_string)
{
	LPTSTR ret = halloc(256);
	HINSTANCE hInst = GetModuleHandle(NULL);
	int n;

	{
		TCHAR tmp[3];

		if(!LoadString(hInst, id_string, tmp, 3))
			return (LPTSTR)TEXT("<<NO STRING RESOURCE FOUND>>");
	}

	if(LoadedStrings)
	{
		STRINGS* s;

		for(s = LoadedStrings; s; s = s->next)
		{
			if(s->id == id_string)
				return s->str;
		}
	}

	for(n = 0; ret; n++, ret = hrealloc(ret, 256 + n * 256))
	{
		int copied;

		memset(ret + (n * 256), '\0', 256);	// hrealloc̓f[^ێB

		copied = LoadString(hInst, id_string, ret, 256 + n * 256);
		if(copied < 256 + n * 256 - 1)
		{
			// ʂȃRXgĂ݂Ȃ
			/*
			char *pc = ret + 256 + n * 256;
			for(; pc != ret && *pc == '\0'; pc--)
				;
			if(*pc)
				pc++;
			ret = hrealloc(ret, pc - ret + 1);
			*/
			break;
		}
	}

	{
		STRINGS* s;

		if(LoadedStrings)
			for(s = LoadedStrings; s->next; s = s->next)
				;

		if(LoadedStrings)
		{
			if(!(s->next = halloc(sizeof(STRINGS))))
			{
				hfree(ret);
				return NULL;
			}
			s = s->next;
		}else
		{
			if(!(LoadedStrings = s = halloc(sizeof(STRINGS))))
			{
				hfree(ret);
				return NULL;
			}
		}

		s->id = id_string;
		s->str = ret;
		s->next = NULL;
	}

	return ret;
}

void FreeLoadedStrings(void)
{
	if(LoadedStrings)
	{
		STRINGS* s;

		for(s = LoadedStrings; s; s = s->next)
			hfree(s->str);
	}

	return;
}

// see http://www.hidecnet.ne.jp/~sinzan/tips/c/c_tip2b.htm
LPCTSTR lstrstri(LPCTSTR string, LPCTSTR pattern)
{
	int length;
	LPCTSTR p;

	// pattern 󕶎̏ꍇ string Ԃ܂B
	if(pattern[0] == TEXT('\0'))
		return string;

	// string ɍŏɌꂽ pattern ւ̃|C^Ԃ܂B
	length = lstrlen(pattern);
	for(p = string; p = lstrchri(p, pattern[0]); p = CharNext(p))
	{
		if(!lstrcmpni(p, pattern, length))
			return p;
	}

	// pattern Ȃꍇ NULL Ԃ܂B
	return NULL;
}

DWORD SetLastErrorReturn(DWORD err)
{
	SetLastError(err);

	return err;
}

DWORD GetDlgItemTextLength(HWND hDlg, UINT id)
{
	return SendMessage(GetDlgItem(hDlg, id), WM_GETTEXTLENGTH, 0, 0);
}

LPTSTR AllocGetDlgItemText(HWND hDlg, UINT id)
{
	LPTSTR str;
	DWORD size;

	size = GetDlgItemTextLength(hDlg, id) + 1 * 2;
	if(!(str = halloc(size)))	// Characters...DBCS...
		return NULL;

	GetDlgItemText(hDlg, id, str, size);

	return str;
}

// TODO: move to registry.c
HKEY getHKeyFromString(LPCTSTR path)
{
	TCHAR hive[32];
	LPCTSTR p;

	p = lstrchr(path, TEXT('\\'));

	if(!p)
		//return NULL;
		p = path + lstrlen(path);

	lstrcpyn(hive, path, ((p - path < 30) ? p - path : 30) + 1);

	if(!lstrcmpi(hive, TEXT("HKCR")) || !lstrcmpi(hive, TEXT("HKEY_CLASSES_ROOT")))		return HKEY_CLASSES_ROOT;
	if(!lstrcmpi(hive, TEXT("HKCU")) || !lstrcmpi(hive, TEXT("HKEY_CURRENT_USER")))		return HKEY_CURRENT_USER;
	if(!lstrcmpi(hive, TEXT("HKLM")) || !lstrcmpi(hive, TEXT("HKEY_LOCAL_MACHINE")))	return HKEY_LOCAL_MACHINE;
	if(!lstrcmpi(hive, TEXT("HKU")) || !lstrcmpi(hive, TEXT("HKEY_USERS")))				return HKEY_USERS;

	// HKEY_PERFORMANCE_DATA HKEY_CURRENT_CONFIG HKEY_DYN_DATA are have not been supported.

	return NULL;
}

LPCTSTR CharNextX(LPCTSTR str)
{
	if(!str)
		return NULL;
	else
		return CharNext(str);
}
