/*
 * teditor.c
 *  BackMatic2 Template Editor
 *   by Murachue
 * 2006/3/30 Create
 */

#include "teditor.h"
#include "owsabox.h"
#include "rkeyslct.h"

#define LI_UNKNOWN	0
#define LI_FILE		1
#define LI_SUBDIR	2
#define LI_REG		3
#define LI_RELATION	4
#define LI_INI		5

int getIconIndexFromCmd(TCHAR cmd)
{
	int idx;

	switch(cmd)
	{
	case TEXT('F'):
		idx = LI_FILE;
		break;
	case TEXT('S'):
		idx = LI_SUBDIR;
		break;
	case TEXT('R'):
		idx = LI_REG;
		break;
	case TEXT('X'):
		idx = LI_RELATION;
		break;
	case TEXT('I'):
		idx = LI_INI;
		break;
	default:
		idx = LI_UNKNOWN;
		break;
	}

	return idx;
}

LPTSTR GetListViewItemText(HWND hWnd, int id, int subid)
{
	LPTSTR tx;
	int tsz = 256;

	while(1)
	{
		tx = halloc(tsz * sizeof(TCHAR));
		ListView_GetItemText(hWnd, id, subid, tx, tsz * sizeof(TCHAR));
		tx[tsz - 1] = '\0'; // 2003.09.16ǉ
		if(lstrlen(tx) != tsz - 1)
			break;

		hfree(tx);
		tsz *= 2;
	}

	return tx;
}

void SwapListViewItem(HWND hWnd, int id1, int id2)
{
	LV_ITEM lvi1, lvi2;

	lvi1.mask = lvi2.mask = LVIF_IMAGE | LVIF_PARAM | LVIF_STATE;
	lvi1.iItem = id1;
	lvi2.iItem = id2;
	lvi1.iSubItem = lvi2.iSubItem = 0;
	lvi1.stateMask = lvi2.stateMask = 0xFFFFFFFF;

	ListView_GetItem(hWnd, &lvi1);
	ListView_GetItem(hWnd, &lvi2);
	lvi1.pszText = GetListViewItemText(hWnd, id1, 0);
	lvi2.pszText = GetListViewItemText(hWnd, id2, 0);

	// swap
	lvi1.iItem = id2;
	lvi2.iItem = id1;

	lvi1.mask |= LVIF_TEXT;
	lvi2.mask |= LVIF_TEXT;

	ListView_SetItem(hWnd, &lvi1);
	ListView_SetItem(hWnd, &lvi2);

	hfree(lvi1.pszText);
	hfree(lvi2.pszText);

	return;
}

typedef struct{
	LPCTSTR tfname;
	BOOL edited;
}TEWNDVAL;

static BOOL CALLBACK TEWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	case WM_INITDIALOG:
		{
			RECT rect;
			TEWNDVAL* tewv;

			//SetWindowLong(GetDlgItem(hWnd, IDC_TLIST), GWL_STYLE, GetWindowLong(GetDlgItem(hWnd, IDC_TLIST), GWL_STYLE) | LVS_NOCOLUMNHEADER);

			ListView_SetExtendedListViewStyle(GetDlgItem(hWnd, IDC_TLIST), ListView_GetExtendedListViewStyle(GetDlgItem(hWnd, IDC_TLIST)) | LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);
			GetClientRect(GetDlgItem(hWnd, IDC_TLIST), &rect);

			{
				HIMAGELIST hil;
				HICON hi;

				hil = ImageList_Create(16, 16, ILC_COLOR4 | ILC_MASK, 6, 1);

				hi = LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_UNKNOWN), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
				ImageList_AddIcon(hil, hi);
				DestroyIcon(hi);

				hi = LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_FILES), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
				ImageList_AddIcon(hil, hi);
				DestroyIcon(hi);

				hi = LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_FOLDER), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
				ImageList_AddIcon(hil, hi);
				DestroyIcon(hi);

				hi = LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_REG), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
				ImageList_AddIcon(hil, hi);
				DestroyIcon(hi);

				hi = LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_RELATION), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
				ImageList_AddIcon(hil, hi);
				DestroyIcon(hi);

				hi = LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_INI), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
				ImageList_AddIcon(hil, hi);
				DestroyIcon(hi);

				ListView_SetImageList(GetDlgItem(hWnd, IDC_TLIST), hil, LVSIL_SMALL);
			}

			{
				LV_COLUMN lvc = {0};
				lvc.mask = LVCF_TEXT | LVCF_WIDTH;
				lvc.pszText = (LPTSTR)AllocLoadString(IDS_CAPTION_TASK);
				lvc.cx = rect.right;
				ListView_InsertColumn(GetDlgItem(hWnd, IDC_TLIST), 0, &lvc);
			}

			tewv = halloc(sizeof(TEWNDVAL));
			tewv->tfname = (LPCTSTR)lParam;
			tewv->edited = FALSE;
			SetWindowLong(hWnd, DWL_USER, (LPARAM)tewv);

			if(lParam)
			{
				LPCTSTR tfname = (LPCTSTR)lParam;
				HANDLE hf;

				if((hf = CreateFile(tfname, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE)
				{
					MessageBox(hWnd, AllocLoadString(IDS_ERROR_CANTOPENTEMPLATE), AllocLoadString(IDS_CAPTION_ERROR), MB_ICONERROR | MB_OK);
					return FALSE;
				}else
				{
					LPTSTR tmpr;
					DWORD read;

					tmpr = halloc(GetFileSize(hf, NULL));
					if(!ReadFile(hf, tmpr, GetFileSize(hf, NULL), &read, NULL))
						read = 0;
					if(read < GetFileSize(hf, NULL))
					{
						MessageBox(hWnd, AllocLoadString(IDS_ERROR_CANTREADTEMPLATE), AllocLoadString(IDS_CAPTION_ERROR), MB_ICONERROR | MB_OK);
					}else
					{
						LPTSTR p = tmpr, q;

						SetDlgItemText(hWnd, IDC_TNAME, GetLine(&p));
						SetDlgItemText(hWnd, IDC_TDESC, GetLine(&p));
						SetDlgItemText(hWnd, IDC_TDEST, GetLine(&p));

						while(q = GetLine(&p))
						{
							if(*CharNext(q) == TEXT(':'))
							{
								LV_ITEM lvi;

								lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;
								lvi.iItem = ListView_GetItemCount(GetDlgItem(hWnd, IDC_TLIST));
								lvi.iSubItem = 0;
								lvi.pszText = CharNext(CharNext(q));
								lvi.lParam = *q;
								lvi.iImage = getIconIndexFromCmd(*q);
								ListView_InsertItem(GetDlgItem(hWnd, IDC_TLIST), &lvi);
							}
						}
					}
					hfree(tmpr);

					CloseHandle(hf);
				}
			}
		}

		return TRUE;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDOK:
			// stub: Save
			{
				TCHAR tfn[MAX_PATH];
				TEWNDVAL* tewv = (TEWNDVAL*)GetWindowLong(hWnd, DWL_USER);

				tfn[0] = TEXT('\0');

				if(tewv->tfname)
				{	// Editing exist template
					int ret;

					if((ret = OWSABox(hWnd)) == IDCANCEL)
						break;
					if(ret == IDC_OW)
						lstrcpy(tfn, tewv->tfname);
				}

				if(!lstrlen(tfn))
				{
					OPENFILENAME ofn = {0};
					TCHAR tpath[MAX_PATH];

					ofn.lStructSize = sizeof(ofn);
					ofn.hInstance = GetModuleHandle(NULL);
					ofn.hwndOwner = hWnd;
					ofn.lpstrFile = tfn;
					ofn.lpstrFilter = TEXT("Template (*.txt)\0*.txt\0");
					ofn.lpstrTitle = AllocLoadString(IDS_CAPTION_SAVETEMPLATE);
					ofn.nMaxFile = sizeof(tfn);
					ofn.lpstrDefExt = TEXT("txt");
					GetModuleFileName(ofn.hInstance, tpath, sizeof(tpath));
					lstrcpy((LPTSTR)basename(tpath), TEXT("template\\"));
					ofn.lpstrInitialDir = tpath;

reselect:
					if(!GetSaveFileName(&ofn))
						break;	// Cancelled.

					// Ⴓ̂̉߂ƌ(lpstrDefExtȂĂ!!)
					//if(lstrcmpi(tfn + lstrlen(tfn) - lstrlen(TEXT(".txt")), TEXT(".txt")))
					//	lstrcat(tfn, TEXT(".txt"));

					if(GetFileAttributes(tfn) != 0xFFFFFFFF)
					{
						if(MessageBox(hWnd, AllocLoadString(IDS_CONFIRM_OVERWRITETEMPLATE), AllocLoadString(IDS_CAPTION_QUESTION), MB_ICONQUESTION | MB_YESNO) == IDNO)
						{
							tfn[0] = TEXT('\0');
							goto reselect;
						}
					}
				}

				{
					LPTSTR tmpr;
					UINT tmprsize;
					int i, max;
					HWND hLst = GetDlgItem(hWnd, IDC_TLIST);
					LPTSTR tx;

					newString(&tmpr, &tmprsize);

					// TODO: append title etc.
					tx = AllocGetDlgItemText(hWnd, IDC_TNAME);
					appendString(&tmpr, &tmprsize, tx);
					appendString(&tmpr, &tmprsize, TEXT("\r\n"));
					hfree(tx);
					tx = AllocGetDlgItemText(hWnd, IDC_TDESC);
					appendString(&tmpr, &tmprsize, tx);
					appendString(&tmpr, &tmprsize, TEXT("\r\n"));
					hfree(tx);
					tx = AllocGetDlgItemText(hWnd, IDC_TDEST);
					appendString(&tmpr, &tmprsize, tx);
					appendString(&tmpr, &tmprsize, TEXT("\r\n"));
					hfree(tx);

					max = ListView_GetItemCount(hLst);

					for(i = 0; i < max; i++)
					{
						LV_ITEM lvi;
						LPCTSTR param;

						lvi.mask = LVIF_IMAGE;
						lvi.iItem = i;
						lvi.iSubItem = 0;
						ListView_GetItem(hLst, &lvi);

						GetListViewItemText(hLst, i, 0);

						switch(lvi.iImage)
						{
						default:			appendString(&tmpr, &tmprsize, TEXT("?:"));	break;
						case LI_FILE:		appendString(&tmpr, &tmprsize, TEXT("F:"));	break;
						case LI_SUBDIR:		appendString(&tmpr, &tmprsize, TEXT("S:"));	break;
						case LI_REG:		appendString(&tmpr, &tmprsize, TEXT("R:"));	break;
						case LI_RELATION:	appendString(&tmpr, &tmprsize, TEXT("X:"));	break;
						case LI_INI:		appendString(&tmpr, &tmprsize, TEXT("I:"));	break;
						}

						param = GetListViewItemText(hLst, i, 0);
						appendString(&tmpr, &tmprsize, param);
						hfree((LPTSTR)param);
						appendString(&tmpr, &tmprsize, TEXT("\r\n"));
					}

					// TODO: Export template.
					{
						HANDLE hf;

RetryOpen:
						if((hf = CreateFile(tfn, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE)
						{
							if(MessageBox(hWnd, AllocLoadString(IDS_ERROR_CANTOPENFILETOWRITEANDRETRY), AllocLoadString(IDS_ERROR_CANTOPENFILE), MB_ICONERROR | MB_YESNO) == IDNO)
							{
								hfree(tmpr);
								EndDialog(hWnd, 0);
								return TRUE;
							}else
							{
								goto RetryOpen;
							}
						}else
						{
							DWORD written;
RetryWrite:
							if(!WriteFile(hf, tmpr, lstrlen(tmpr) * sizeof(TCHAR), &written, NULL))
								written = 0;
							if(written < lstrlen(tmpr) * sizeof(TCHAR))
								if(MessageBox(hWnd, AllocLoadString(IDS_ERROR_CANTWRITEFILEANDRETRY), AllocLoadString(IDS_ERROR_CANTOPENFILE), MB_ICONERROR | MB_YESNO) == IDNO)
								{
									CloseHandle(hf);
									hfree(tmpr);
									EndDialog(hWnd, 0);
									return TRUE;
								}else
								{
									SetFilePointer(hf, 0, NULL, FILE_BEGIN);
									goto RetryWrite;
								}
							CloseHandle(hf);
						}
					//MessageBox(hWnd, tmpr, tfn, MB_OK);
					}

					hfree(tmpr);
				}
			}
			EndDialog(hWnd, 1);
			break;
		case IDCANCEL:
			{
				TEWNDVAL* tewv = (TEWNDVAL*)GetWindowLong(hWnd, DWL_USER);
				if(tewv->edited)
				{
					if(MessageBox(hWnd, AllocLoadString(IDS_CONFIRM_CANCELTEMPLATEEDITING), AllocLoadString(IDS_CAPTION_QUESTION), MB_ICONQUESTION | MB_YESNO) == IDYES)
						EndDialog(hWnd, 0);
				}else
				{
					EndDialog(hWnd, 0);
				}
			}
			break;

			{
				int id;

		case IDC_TIADD:
			id = ListView_GetItemCount(GetDlgItem(hWnd, IDC_TLIST));

			{
				LV_ITEM lvi;

				lvi.mask = 0;
				lvi.iItem = id;
				lvi.iSubItem = 0;

				ListView_InsertItem(GetDlgItem(hWnd, IDC_TLIST), &lvi);
			}

			// add id
			goto SetItem;
			break;
		case IDC_TIEDIT:
			{
				HWND hLst = GetDlgItem(hWnd, IDC_TLIST);
				int max = ListView_GetItemCount(hLst);
				int i = -1;
				int count = 0;

				while((i = ListView_GetNextItem(hLst, i, LVNI_SELECTED)) != -1)
					count++;

				if(count == 0)
				{
					MessageBox(hWnd, AllocLoadString(IDS_NOTIFY_NOSELECTED), AllocLoadString(IDS_CAPTION_NOTIFY), MB_ICONNOTIFY | MB_OK);
				}
				if(count > 1)
				{
					MessageBox(hWnd, AllocLoadString(IDS_NOTIFY_TWOORMORESELECTED), AllocLoadString(IDS_CAPTION_NOTIFY), MB_ICONNOTIFY | MB_OK);
				}
				if(count == 1)
				{
					id = ListView_GetNextItem(hLst, -1, LVNI_SELECTED);
					goto SetItem;
				}
			}
			break;
SetItem:
			{
				HWND hLst = GetDlgItem(hWnd, IDC_TLIST);
				LV_ITEM lvi;

				lvi.mask = LVIF_TEXT | LVIF_IMAGE;
				lvi.iItem = id;
				lvi.iSubItem = 0;

				// stub: set id
				if(SendMessage(GetDlgItem(hWnd, IDC_RFILE), BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					// File
					if(SendMessage(GetDlgItem(hWnd, IDC_INCSUBDIR), BM_GETCHECK, 0, 0) == BST_CHECKED)
					{
						// Subdir
						TCHAR fn[MAX_PATH];

						GetDlgItemText(hWnd, IDC_PATH, fn, sizeof(fn));
						if(lstrlen(fn))
						{
							lvi.iImage = LI_SUBDIR;
							lvi.pszText = fn;
							ListView_SetItem(hLst, &lvi);
						}
					}else
					{
						// OnlyFile
						TCHAR fn[MAX_PATH];

						GetDlgItemText(hWnd, IDC_PATH, fn, sizeof(fn));
						if(lstrlen(fn))
						{
							lvi.iImage = LI_FILE;
							lvi.pszText = fn;
							ListView_SetItem(hLst, &lvi);
						}
					}
				}
				if(SendMessage(GetDlgItem(hWnd, IDC_RREG), BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					// Reg
					TCHAR ky[MAX_PATH];

					GetDlgItemText(hWnd, IDC_KEY, ky, sizeof(ky));
					if(lstrlen(ky))
					{
						lvi.iImage = LI_REG;
						lvi.pszText = ky;
						ListView_SetItem(hLst, &lvi);
					}
				}
				if(SendMessage(GetDlgItem(hWnd, IDC_RRELATION), BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					// Relation
					TCHAR ef[MAX_PATH];

					GetDlgItemText(hWnd, IDC_EXEFILE, ef, sizeof(ef));
					if(lstrlen(ef))
					{
						lvi.iImage = LI_RELATION;
						lvi.pszText = ef;
						ListView_SetItem(hLst, &lvi);
					}
				}
				if(SendMessage(GetDlgItem(hWnd, IDC_RINI), BM_GETCHECK, 0, 0) == BST_CHECKED)
				{
					// Ini
					lvi.iImage = LI_INI;
					lvi.pszText = TEXT("(INI)");
					ListView_SetItem(hLst, &lvi);
					MessageBox(hWnd, TEXT("Ini is not supported yet."), AllocLoadString(IDS_CAPTION_ERROR), MB_OK);
				}

				{
					TEWNDVAL* tewv = (TEWNDVAL*)GetWindowLong(hWnd, DWL_USER);
					tewv->edited = TRUE;
				}
			}
			break;
			}
		case IDC_TIDELETE:
			{
				UINT selected = 0;
				int max, i;
				HWND hLst;

				hLst = GetDlgItem(hWnd, IDC_TLIST);

				max = ListView_GetItemCount(hLst);

				for(i = 0; i < max; i++)
				{
					if(ListView_GetItemState(hLst, i, LVIS_SELECTED) & LVIS_SELECTED)
						selected++;
				}

				if(!selected)
				{
					MessageBox(hWnd, AllocLoadString(IDS_NOTIFY_NOSELECTED), AllocLoadString(IDS_CAPTION_NOTIFY), MB_ICONNOTIFY | MB_OK);
				}else
				{
					TCHAR ts[1024];

					wsprintf(ts, AllocLoadString(IDS_CONFIRM_DELETETASKS), selected);
					if(MessageBox(hWnd, ts, AllocLoadString(IDS_CAPTION_QUESTION), MB_ICONWARNING | MB_YESNO | MB_DEFBUTTON2) == IDYES)
					{
						int i;

						for(i = 0; i < ListView_GetItemCount(hLst); i++)
						{
							if(ListView_GetItemState(hLst, i, LVIS_SELECTED) & LVIS_SELECTED)
							{
								ListView_DeleteItem(hLst, i);
								i--;
								{
									TEWNDVAL* tewv = (TEWNDVAL*)GetWindowLong(hWnd, DWL_USER);
									tewv->edited = TRUE;
								}
							}
						}
					}
				}
			}
			break;
		case IDC_TIMOVEUP:
			{
				int max, i;
				HWND hLst = GetDlgItem(hWnd, IDC_TLIST);

				max = ListView_GetItemCount(hLst);

				for(i = 1; i < max; i++)
				{
					if(ListView_GetItemState(hLst, i, LVIS_SELECTED) & LVIS_SELECTED
						&&
						!(ListView_GetItemState(hLst, i - 1, LVIS_SELECTED) & LVIS_SELECTED)
						)
					{
						SwapListViewItem(hLst, i - 1, i);
						{
							TEWNDVAL* tewv = (TEWNDVAL*)GetWindowLong(hWnd, DWL_USER);
							tewv->edited = TRUE;
						}
					}
				}
			}
			break;
		case IDC_TIMOVEDOWN:
			{
				int max, i;
				HWND hLst = GetDlgItem(hWnd, IDC_TLIST);

				max = ListView_GetItemCount(hLst);

				for(i = max - 2; i >= 0; i--)	// tɂȂƃhhɉ
				{
					if(ListView_GetItemState(hLst, i, LVIS_SELECTED) & LVIS_SELECTED
						&&
						!(ListView_GetItemState(hLst, i + 1, LVIS_SELECTED) & LVIS_SELECTED)
						)
					{
						SwapListViewItem(hLst, i, i + 1);
						{
							TEWNDVAL* tewv = (TEWNDVAL*)GetWindowLong(hWnd, DWL_USER);
							tewv->edited = TRUE;
						}
					}
				}
			}
			break;

		case IDC_RFILE:
			// File
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_PATH), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_PATH), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERDIR), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_INCSUBDIR), TRUE);
			// Reg
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_KEY), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_KEY), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERREG), FALSE);
			// Rel
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_EXE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_EXEFILE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFEREXE), FALSE);
			// Ini
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INI), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIFILE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERINI), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INISECTION), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INISECTION), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INIVALUE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIVALALL), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIVALUE), FALSE);
			break;
		case IDC_RREG:
			// File
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_PATH), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_PATH), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERDIR), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INCSUBDIR), FALSE);
			// Reg
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_KEY), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_KEY), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERREG), TRUE);
			// Rel
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_EXE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_EXEFILE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFEREXE), FALSE);
			// Ini
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INI), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIFILE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERINI), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INISECTION), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INISECTION), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INIVALUE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIVALALL), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIVALUE), FALSE);
			break;
		case IDC_RRELATION:
			// File
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_PATH), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_PATH), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERDIR), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INCSUBDIR), FALSE);
			// Reg
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_KEY), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_KEY), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERREG), FALSE);
			// Rel
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_EXE), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_EXEFILE), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFEREXE), TRUE);
			// Ini
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INI), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIFILE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERINI), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INISECTION), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INISECTION), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INIVALUE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIVALALL), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIVALUE), FALSE);
			break;
		case IDC_RINI:
			// File
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_PATH), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_PATH), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERDIR), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_INCSUBDIR), FALSE);
			// Reg
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_KEY), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_KEY), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERREG), FALSE);
			// Rel
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_EXE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_EXEFILE), FALSE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFEREXE), FALSE);
			// Ini
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INI), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIFILE), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_REFERINI), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INISECTION), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_INISECTION), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_STATIC_INIVALUE), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIVALALL), TRUE);
			EnableWindow(GetDlgItem(hWnd, IDC_INIVALUE), TRUE);
			break;

		case IDC_REFERDIR:
			//MessageBox(hWnd, "ReferDir", "debug", MB_OK);
			{
				BROWSEINFO bi = {0};
				LPITEMIDLIST id;

				bi.hwndOwner = hWnd;
				bi.lpszTitle = AllocLoadString(IDS_SELECTDIR);
				bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_DONTGOBELOWDOMAIN/* | 0x0040*/;
				if(id = SHBrowseForFolder(&bi))
				{
					char buf[MAX_PATH];

					SHGetPathFromIDList(id, buf);
					CoTaskMemFree(id);
					addEndYen(buf);
					lstrcat(buf, TEXT("*.*"));
					SetDlgItemText(hWnd, IDC_PATH, buf);
				}
			}
			break;
		case IDC_REFERREG:
			//MessageBox(hWnd, "ReferReg", "debug", MB_OK);
			{
				LPTSTR rkey;

				if(rkey = RegKeySelectDlg(hWnd))
				{
					SetDlgItemText(hWnd, IDC_KEY, rkey);
					hfree(rkey);
				}
			}
			break;
		case IDC_REFEREXE:
			//MessageBox(hWnd, "ReferExe", "debug", MB_OK);
			{
				OPENFILENAME ofn = {0};
				char buf[MAX_PATH];

				ofn.lStructSize = sizeof(ofn);
				ofn.Flags = OFN_FILEMUSTEXIST;
				ofn.hwndOwner = hWnd;
				ofn.lpstrFilter = TEXT("st@C(*.exe)\0*.exe\0St@C(*.*)\0*.*\0");
				ofn.lpstrCustomFilter = NULL;
				ofn.nFilterIndex = 1;
				buf[0] = 0;
				ofn.lpstrFile = (LPSTR)buf;
				ofn.nMaxFile = sizeof(buf);
				ofn.lpstrInitialDir = NULL;
				ofn.lpstrTitle = AllocLoadString(IDS_CAPTION_FILESAVE);
				ofn.lpstrDefExt = TEXT("exe");
				if(GetOpenFileName((LPOPENFILENAME)&ofn))
					SetDlgItemText(hWnd, IDC_EXEFILE, buf);
			}
			break;
		case IDC_REFERINI:
			//MessageBox(hWnd, "ReferIni", "debug", MB_OK);
			{
				OPENFILENAME ofn = {0};
				char buf[MAX_PATH];

				ofn.lStructSize = sizeof(ofn);
				ofn.Flags = OFN_FILEMUSTEXIST;
				ofn.hwndOwner = hWnd;
				ofn.lpstrFilter = TEXT("INIt@C(*.ini)\0*.ini\0St@C(*.*)\0*.*\0");
				ofn.lpstrCustomFilter = NULL;
				ofn.nFilterIndex = 1;
				buf[0] = 0;
				ofn.lpstrFile = (LPSTR)buf;
				ofn.nMaxFile = sizeof(buf);
				ofn.lpstrInitialDir = NULL;
				ofn.lpstrTitle = AllocLoadString(IDS_CAPTION_FILESAVE);
				ofn.lpstrDefExt = TEXT("ini");
				if(GetOpenFileName((LPOPENFILENAME)&ofn))
					SetDlgItemText(hWnd, IDC_INIFILE, buf);
			}
			break;
		}
		break;
	/*
	case WM_CLOSE:
		SendMessage(hWnd, WM_COMMAND, IDCANCEL, 0);
		break;
	*/
	case WM_DESTROY:
		ImageList_Destroy(ListView_GetImageList(GetDlgItem(hWnd, IDC_TLIST), LVSIL_SMALL));
		{
			TEWNDVAL* tewv = (TEWNDVAL*)GetWindowLong(hWnd, DWL_USER);
			hfree(tewv);
		}
		break;
	}

	return FALSE;
}

int CreateTEditForm(HWND hWndParent, LPCTSTR tfname)
{
	return DialogBoxParam((HINSTANCE)GetWindowLong(hWndParent, GWL_HINSTANCE), MAKEINTRESOURCE(IDD_TEDIT), hWndParent, (DLGPROC)TEWndProc, (LPARAM)tfname);
}
